from typing import Tuple, List

import pygame


pygame.font.init()
FONT = pygame.font.SysFont("Arial", 25)


class BoxFullError(Exception):
    pass


class BoxIsLockedError(Exception):
    pass


class Box:
    def __init__(self, x: int, y: int, max_capacity: int) -> None:
        """Initializes a box.

        Args:
            x: The X postition of the box
            y: The Y position of the box
            max_capacity: The maximum number of objects that can be stored in the box
        """
        self.x = x
        self.y = y
        self.max_capacity = max_capacity

        self._contents = []
        self.width = 75
        self.height = 50


    def draw(self, screen: pygame.Surface) -> None:
        rect = (self.x, self.y, self.width, self.height)
        pygame.draw.rect(screen, (181, 101, 29), rect)

        # draw capacity text
        display_text = "{}/{}".format(len(self._contents), self.max_capacity)
        text = FONT.render(display_text, True, (0, 0, 0))
        screen.blit(text, (self.x, self.y))
    
    def is_clicked(self, mouse_pos) -> bool:  # IS CLICKED
        rect = pygame.Rect(self.x, self.y, self.width, self.height)
        return bool(rect.collidepoint(mouse_pos))

    def add_item(self, item: int) -> None:
        if len(self._contents) == self.max_capacity:
            raise BoxFullError("Cannot add item to a full box.")
        
        self._contents.append(1)



class LockableBox(Box):
    def __init__(self, x: int, y: int, max_capacity: int):
        super().__init__(x, y, max_capacity)

        # additional attribute
        self._locked = True
    

    def add_item(self, item: int) -> None:
        if self._locked is True:
            raise BoxIsLockedError("Cannot add item to a locked box.")

        super().add_item(item)
    
    def draw(self, screen):
        super().draw(screen)

        if self._locked is True:
            text = FONT.render("Locked", True, (0, 0, 0))
            screen.blit(text, (self.x, self.y + self.height//2))
        
    def toggle_lock(self):
        self._locked = not self._locked


